LegionHail.ui = {}

LegionHail.ui.main_dlg = nil
LegionHail.ui.main_list = nil
LegionHail.ui.delete_button = nil
LegionHail.ui.delete_mode_button = nil
LegionHail.ui.main_delete_hail = nil
LegionHail.ui.current = nil
LegionHail.ui.mode = 1
LegionHail.ui.mode_text = nil


-- Show the main gui
function LegionHail.ui.show()
	if LegionHail.ui.main_dlg == nil then
		LegionHail.ui.CreateDlg()
	end
	ShowDialog(LegionHail.ui.main_dlg,iup.CENTER, iup.CENTER)
end


-- Update the dialog box
function LegionHail.ui.update()
	LegionHail.ui.mode_text.title = LegionHail.get_mode(LegionHail.ui.mode).name
	
	--update the list of hails
	local i = 1
	for _,hail in pairs(LegionHail.data.modes[LegionHail.ui.mode].list) do
		LegionHail.ui.main_list[i] = hail.msg
		i = i + 1
		LegionHail.ui.main_list[i] = nil
	end
	
	--only enable the delete_button if there are more than one hails
	--if #LegionHail.data.modes[LegionHail.ui.mode].list > 1 then
	if i > 2 then
		LegionHail.ui.delete_button.active = "yes"
	else
		LegionHail.ui.delete_button.active = "no"
	end
	
	--only enable the delete_mode_button if there are more than one modes
	if #LegionHail.data.modes > 1 then
		LegionHail.ui.delete_mode_button.active = "yes"
	else
		LegionHail.ui.delete_mode_button.active = "no"
	end
end


-- Pop up a box to edit the hail of LegionHail.ui.current (or create a new one if "new" is true)
function LegionHail.ui.edit(new)
	local hail = {}
	if new then
		hail.msg = "New Hail"
		hail.weight = 1
	else
		hail = LegionHail.get_hail(LegionHail.ui.current, LegionHail.ui.mode)
	end
	local editdlg
	local editv = iup.vbox{
		iup.hbox{iup.fill{},iup.label{title="Update Hail",font=Font.H3},iup.fill{}},
		gap=2,
		margin="2x2"
	}
	local text = iup.text{value = hail.msg, size="925"}
	local weight = iup.text{value = hail.weight, size="50"}
	local apply_button = iup.stationbutton{title="Apply",action=function()
			if new then
				LegionHail.add_hail(text.value, weight.value, LegionHail.ui.mode)
			else
				LegionHail.update_hail(LegionHail.ui.current, text.value, weight.value, LegionHail.ui.mode)
			end
			LegionHail.ui.current = nil
			LegionHail.ui.update()
			HideDialog(editdlg)
		end}
	
	local cancel_button = iup.stationbutton{title="Cancel",action=function() HideDialog(editdlg) end}	

	iup.Append(editv, iup.hbox{iup.label{title="Text: "}, text, alignment="ACENTER",gap=2})
	iup.Append(editv, iup.hbox{iup.label{title="Weight: "}, weight, alignment="ACENTER",gap=2})
	iup.Append(editv, iup.hbox{iup.fill{}, weight, iup.fill{}})
	iup.Append(editv, iup.hbox{iup.fill{}, cancel_button, apply_button, gap=2})

	--the actual dialog
	editdlg = iup.dialog{
		iup.stationhighopacityframe{
			iup.stationhighopacityframebg{
				editv
			}
		},
		menubox="NO", resize="NO", border="NO", topmost="YES",
		bgcolor = "0 0 0 0 *",
		size="1020x200",
		defaultenter = apply_button,
		defaultesc = cancel_button,
	}
	ShowDialog(editdlg,iup.CENTER, iup.CENTER)
end


-- New hail
function LegionHail.ui.new()
	LegionHail.ui.edit(true)
end


-- Delete LegionHail.ui.current
function LegionHail.ui.delete()
	LegionHail.remove_hail(LegionHail.ui.current, LegionHail.ui.mode)
	LegionHail.ui.update()
	LegionHail.ui.current = nil
end


-- Pop up a box to add a mode if a parameter is given, otherwise rename the mode spec'ed by LegionHail.ui.mode
function LegionHail.ui.update_mode(new_mode)
	local mode = {}
	if not new_mode then
		mode = LegionHail.get_mode(LegionHail.ui.mode)
	else
		mode.list = {}
		mode.name = "Mode "..new_mode
	end
	local updatedlg
	local updatev = iup.vbox{
		iup.hbox{iup.fill{},iup.label{title="Update Mode",font=Font.H3},iup.fill{}},
		gap=2,
		margin="2x2"
	}
	local text = iup.text{value = mode.name, size="925"}
	local apply_button = iup.stationbutton{title="Apply",action=function()
			if new_mode then
				LegionHail.add_mode(new_mode, text.value)
				LegionHail.add_hail(LegionHail.default_hail, 1, new_mode)
				LegionHail.ui.mode = new_mode
			else
				LegionHail.add_mode(LegionHail.ui.mode, text.value)
			end
			LegionHail.ui.update()
			HideDialog(updatedlg)
		end}
	
	local cancel_button = iup.stationbutton{title="Cancel",action=function() HideDialog(updatedlg) end}	

	iup.Append(updatev, iup.hbox{iup.label{title="Text: "}, text, alignment="ACENTER",gap=2})
	iup.Append(updatev, iup.hbox{iup.fill{}, cancel_button, apply_button, gap=2})

	--the actual dialog
	updatedlg = iup.dialog{
		iup.stationhighopacityframe{
			iup.stationhighopacityframebg{
				updatev
			}
		},
		menubox="NO", resize="NO", border="NO", topmost="YES",
		bgcolor = "0 0 0 0 *",
		size="1020x200",
		defaultenter = apply_button,
		defaultesc = cancel_button,
	}
	ShowDialog(updatedlg,iup.CENTER, iup.CENTER)
end


-- Pop up a box to confirm the deletion of mode_id (or LegionHail.ui.mode if none specified)
function LegionHail.ui.delete_mode(mode_id)
	if not mode_id then
		mode_id = LegionHail.ui.mode
	end
	local mode = LegionHail.get_mode(mode_id)
	local updatedlg
	local updatev = iup.vbox{
		iup.hbox{iup.fill{},iup.label{title="Confirm Mode Deletion",font=Font.H3},iup.fill{}},
		gap=2,
		margin="2x2"
	}
	
	local label = iup.label{title="Are you sure you wish to delete mode '"..mode.name.."'?"};
	local apply_button = iup.stationbutton{title="Delete",action=function()
			LegionHail.remove_mode(mode_id)
			if LegionHail.ui.mode > #LegionHail.data.modes then
				LegionHail.ui.mode = #LegionHail.data.modes
			end
			LegionHail.ui.update()
			HideDialog(updatedlg)
		end}
	
	local cancel_button = iup.stationbutton{title="Cancel",action=function() HideDialog(updatedlg) end}	

	iup.Append(updatev, iup.hbox{iup.label{title="Text: "}, label, alignment="ACENTER",gap=2})
	iup.Append(updatev, iup.hbox{iup.fill{}, cancel_button, apply_button, gap=2})

	--the actual dialog
	updatedlg = iup.dialog{
		iup.stationhighopacityframe{
			iup.stationhighopacityframebg{
				updatev
			}
		},
		menubox="NO", resize="NO", border="NO", topmost="YES",
		bgcolor = "0 0 0 0 *",
		size="1020x200",
		defaultenter = apply_button,
		defaultesc = cancel_button,
	}
	ShowDialog(updatedlg,iup.CENTER, iup.CENTER)
end


-- Create the main dialog
function LegionHail.ui.CreateDlg()
	--crate main vbox to hold everything
	local mainv = iup.vbox{
		iup.hbox{iup.fill{},iup.label{title="\127FF8800LegionHail",font=Font.H3},iup.fill{}},
		gap=2,
		margin="2x2"
	}
	
	--create the mode setter thingie
	local mode_last_button = iup.stationbutton{title="<",action=function() LegionHail.ui.mode = (LegionHail.ui.mode - 2) % #LegionHail.data.modes + 1; LegionHail.ui.update() end}
	local mode_next_button = iup.stationbutton{title=">",action=function() LegionHail.ui.mode = LegionHail.ui.mode % #LegionHail.data.modes + 1; LegionHail.ui.update() end}
	LegionHail.ui.mode_text = iup.label{title=LegionHail.get_mode(LegionHail.ui.mode).name, expand="HORIZONTAL"};
	local rename_mode_button = iup.stationbutton{title="Rename Mode",action=function() LegionHail.ui.update_mode() end}
	local add_mode_button = iup.stationbutton{title="New Mode",action=function() LegionHail.ui.update_mode(#LegionHail.data.modes + 1) end}
	LegionHail.ui.delete_mode_button = iup.stationbutton{title="Delete Mode",action=function() LegionHail.ui.delete_mode() end}
	iup.Append(mainv, iup.hbox{mode_last_button, mode_next_button, iup.fill{size=20}, LegionHail.ui.mode_text, iup.fill{}, rename_mode_button, add_mode_button, LegionHail.ui.delete_mode_button, alignment="ACENTER",gap=2})


	--create a list of hails
	LegionHail.ui.main_list = iup.list({expand="YES", bgcolor="37 73 98"})  --254962
	--set action when clicked/double-clicked
	function LegionHail.ui.main_list:action(title,i,v)
		--TODO: set timer for better dblclk detection
		if v == 1 then
			if title == LegionHail.ui.current then
				LegionHail.ui.edit()
			else
				LegionHail.ui.current=title
			end
		end
	end
	iup.Append(mainv, LegionHail.ui.main_list)
	local edit_button = iup.stationbutton{title="Edit Hail",action=function() if LegionHail.ui.current then LegionHail.ui.edit() end end}
	local new_button = iup.stationbutton{title="New Hail",action=function() LegionHail.ui.new() end}
	LegionHail.ui.delete_button = iup.stationbutton{title="Delete Hail",action=function() if LegionHail.ui.current then LegionHail.ui.delete() end end}
	
	local close_button = iup.stationbutton{title="Close",action=function() HideDialog(LegionHail.ui.main_dlg) end}	

	iup.Append(mainv, iup.hbox{edit_button, new_button, LegionHail.ui.delete_button, iup.fill{}, close_button, alignment="ACENTER",gap=2})
	
	LegionHail.ui.update()
	
	LegionHail.ui.main_dlg = iup.dialog{
		iup.stationhighopacityframe{
			iup.stationhighopacityframebg{
				mainv
			}
		},
		menubox="NO", resize="NO", border="NO", topmost="YES",
		bgcolor = "0 0 0 0 *",
		size="THREEQUARTERxHALF",
		defaultesc = close_button,
	}

end
