declare('LegionHail', LegionHail or {})
LegionHail.version = 1.0
math.randomseed( os.time() );

--we can optionally use the KombatTunes plugin, so declare it now so that we can check for its existence laster
declare('KombatTunes', KombatTunes or {})

--set up some defaults
LegionHail.data = {}
LegionHail.save_id = 64800185674
LegionHail.current_mode = 1
LegionHail.default_hail = "Hail"

dofile("gui.lua")

-- Load data
function LegionHail.load(force)
	if force or not LegionHail.data.modes then
		LegionHail.data = unspickle(LoadSystemNotes(LegionHail.save_id))
	end
	--if the data is empty, initialize it quick
	if not LegionHail.data.modes then
		LegionHail.add_mode(1)
		LegionHail.add_hail(LegionHail.default_hail, 1, 1)
	end
end


-- Save data
function LegionHail.save()
	SaveSystemNotes(spickle(LegionHail.data), LegionHail.save_id)
end


-- Hail somebody
function LegionHail.hail()
	local hail = LegionHail.select_weighted_random()
	LegionHail.send(hail.msg)
	--TODO: Make KombatTunes optional and mode/hail based
	--      Would be really nice if it would play certain songs with certain hails as well,
	--      or if there were a way to manually trigger that



	if KombatTunes then
		if KombatTunes.version then
			if KombatTunes.version >= 1.2 then
				KombatTunes.fight()
			end
		end
	end
end


-- Send msg to current target via pm, and echo it back to the chat screen
function LegionHail.send(msg)
	local target = GetTargetInfo()
	if (target ~= nil) then
		--Send the message
		SendChat(msg, "PRIVATE", target)
		print("\12780FF80->"..target..": "..msg)
	end
end


-- Incriment mode
function LegionHail.cycle()
	LegionHail.current_mode = LegionHail.current_mode%#LegionHail.data.modes + 1
	LegionHail.print_mode()
end


-- Display the current mode
function LegionHail.print_mode()
	local mode = LegionHail.get_mode()
	if mode then
		print("\12728B4F0legionmode: "..mode.name)
	end
end


function LegionHail.test_rand(num_tests)
	local n = 0
	local accumulator = {}
	while (n < num_tests) do
		local hail = LegionHail.select_weighted_random()
		if not accumulator[hail.msg] then
			accumulator[hail.msg] = 0
		end
		accumulator[hail.msg] = accumulator[hail.msg] + 1
		n = n + 1
	end
	for i,v in pairs(accumulator) do
		print("Results:")
		print(math.ceil(100*v/n).."% : "..i)
	end
end


-- Select hail at random according to mode
function LegionHail.select_random()
	local n = math.random(LegionHail.count_hails())
	return LegionHail.get_hail_n(n)
end


-- Select hail at random according to mode, considering weights
function LegionHail.select_weighted_random()
	local n = math.random(LegionHail.count_weighted_hails())
	return LegionHail.get_weighted_hail_n(n)
end


-- Count hails in mode, or current_mode if not provided
function LegionHail.count_hails(mode)
	if not mode then mode = LegionHail.current_mode end
	if LegionHail.data.modes and LegionHail.data.modes[mode] and LegionHail.data.modes[mode].list then
		local count = 0
		for _,v in pairs(LegionHail.data.modes[mode].list) do
			count = count + 1
		end
		return count
	else
		return 0
	end
end


-- Count hails in mode, or current_mode if not provided.  Each hail is worth hail.weight counts.
function LegionHail.count_weighted_hails(mode)
	if not mode then mode = LegionHail.current_mode end
	if LegionHail.data.modes and LegionHail.data.modes[mode] and LegionHail.data.modes[mode].list then
		local count = 0
		for _,hail in pairs(LegionHail.data.modes[mode].list) do
			count = count + hail.weight
		end
		return count
	else
		return 0
	end
end


-- Get an existing hail, by msg
function LegionHail.get_hail(msg, mode)
	if LegionHail.data.modes and LegionHail.data.modes[mode] and LegionHail.data.modes[mode].list and LegionHail.data.modes[mode].list[msg] then
		return LegionHail.data.modes[mode].list[msg]
	else
		return nil
	end
end


-- Get an existing hail, by number
function LegionHail.get_hail_n(n, mode)
	local mode = LegionHail.get_mode(mode)
	if mode then
		local i = 1
		for _,hail in pairs(mode.list) do
			if i == n then
				--use this one
				return hail
			end
			i = i + 1
		end
	end	
end


-- Get an existing hail, by number, considering weights
function LegionHail.get_weighted_hail_n(n, mode)
	local mode = LegionHail.get_mode(mode)
	if mode then
		local i = 1
		for _,hail in pairs(mode.list) do
			if n >= i and n < i+hail.weight then
				--use this one
				return hail
			end
			i = i + hail.weight
		end
	end	
end


-- Add new hail
function LegionHail.add_hail(msg, weight, mode)
	if not LegionHail.data.modes then LegionHail.data.modes = {} end
	if not LegionHail.data.modes[mode] then LegionHail.data.modes[mode] = {} end
	if not LegionHail.data.modes[mode].list then LegionHail.data.modes[mode].list = {} end
	if not LegionHail.data.modes[mode].list[msg] then LegionHail.data.modes[mode].list[msg] = {} end
	LegionHail.data.modes[mode].list[msg].msg = msg
	LegionHail.data.modes[mode].list[msg].weight = weight
	LegionHail.save()
end


-- Remove hail
function LegionHail.remove_hail(msg, mode)
	if LegionHail.get_hail(msg, mode) then
		LegionHail.data.modes[mode].list[msg] = nil
	end
	LegionHail.save()
end


-- Update hail
function LegionHail.update_hail(orig_msg, new_msg, weight, mode)
	LegionHail.remove_hail(orig_msg, mode);
	LegionHail.add_hail(new_msg, weight, mode);
end


-- Get mode object by mode number (or current_mode if not provided)
function LegionHail.get_mode(mode)
	if not mode then mode = LegionHail.current_mode end
	if LegionHail.data.modes and LegionHail.data.modes[mode] then
		return LegionHail.data.modes[mode]
	else
		return nil
	end
end


-- Add mode
function LegionHail.add_mode(mode, name)
	if not name then name = "Mode "..mode end
	if not LegionHail.data.modes then LegionHail.data.modes = {} end
	if not LegionHail.data.modes[mode] then LegionHail.data.modes[mode] = {} end
	if not LegionHail.data.modes[mode].list then LegionHail.data.modes[mode].list = {} end
	LegionHail.data.modes[mode].name = name
	LegionHail.save()
end


-- Remove mode
function LegionHail.remove_mode(mode)
	--TODO: deal with what happens if "mode" is not the last index - the others must be reassigned
	if LegionHail.get_mode(mode) then
		while (mode < #LegionHail.data.modes) do
			LegionHail.swap_mode(mode, mode+1)
			mode = mode+1
		end
		LegionHail.data.modes[mode] = nil
	end
	LegionHail.save()
end


-- Swap modes
function LegionHail.swap_mode(mode_a, mode_b)
	if not (LegionHail.get_mode(mode_a) and LegionHail.get_mode(mode_b)) then
		return
	end
	
	local tmp_mode = LegionHail.get_mode(mode_a)
	LegionHail.data.modes[mode_a] = LegionHail.data.modes[mode_b]
	LegionHail.data.modes[mode_b] = tmp_mode
	LegionHail.save()
end







--process the /LegionHail command to decide what to do
function LegionHail.process(_,data)
	LegionHail.load()
	if (data ~= nil and (data[1] == "print" or data[1] == "p")) then
		LegionHail.print_mode();
	elseif (data ~= nil and (data[1] == "cycle" or data[1] == "c")) then
		LegionHail.cycle();
	--elseif (data ~= nil and (data[1] == "fee" or data[1] == "f")) then
	--	LegionHail.set_fee(data[2]);
	elseif (data ~= nil and (data[1] == "gui")) then
		LegionHail.ui.show();
	elseif (data ~= nil and (data[1] == "remove_hail")) then
		if #data == 2 then
			LegionHail.remove_hail(data[2])
		else
			print("/legionhail remove_hail \"msg\"")
		end
	elseif (data ~= nil and (data[1] == "add_hail")) then
		if #data == 4 then
			LegionHail.add_hail(data[2], tonumber(data[3]), tonumber(data[4]))
		else
			print("/legionhail add_hail \"msg\" weight mode")
		end
	elseif (data ~= nil and (data[1] == "add_mode")) then
		if #data == 3 then
			LegionHail.add_mode(tonumber(data[2]), tostring(data[3]))
		else
			print("/legionhail add_mode mode name")
		end
	elseif (data ~= nil and (data[1] == "remove_mode")) then
		if #data == 2 then
			LegionHail.remove_mode(tonumber(data[2]))
		else
			print("/legionhail remove_mode mode")
		end
	else
		LegionHail.hail();
	end
end

RegisterUserCommand("legionhail-cycle",function() LegionHail.process(nil, {"cycle"}) end);
RegisterUserCommand("legionhail",LegionHail.process);
RegisterUserCommand("lh",LegionHail.process);
